--[[
	author: Zax
	content:
		globales: REPUT_
		functions: Reput.
		main UI frame: ReputFrameMain
	todo:
--]]

---------------------------------------------------------------------------------------
------------------------------------- VARIABLES ---------------------------------------
Reput = {} -- nommage espace
local addonName = ...

local addonVar = {
	------------------------ nécessaire !!!
	["name"] = addonName,
	["version"] = GetAddOnMetadata(addonName, "Version"), -- ne pas convertir en nombre ici !
	["chatColor"] = "8A5EC4",
	["slashCmd"] = "reputations",
	["settingsPanel"] = "", -- nom du frame de settings : laisser vide si utilisation du frame de WoW (ou si pas de settings)
	["minimap"] = "", -- nécessite le fichier _minimap.lua (laisser vide ou NIL si pas de bouton minimap)
	------------------------ UI
	["widthUI"] = 765, -- px
	["bkColor"] = "666666", -- fond
	["bkAlpha"] = 0.4, -- fond
	["vLine"] = 15, -- px : hauteur d'une ligne de la liste
	["maxVisibleLines"] = 40, -- pour resize liste scroll
}

function Reput.getLocal(key) -- pour traduction (voir localization.lua) : ne pas deplacer !!!
	if (REPUT_LOCAL_TBL[key] == nil) then return key else return REPUT_LOCAL_TBL[key] end
end

local frameList, frameUI -- scroll frame et frams sous menubar
local playerName, playerFaction, playerServer, playerLevel, playerClass, playerGuild
local exaltedValue = 42000 -- /////
local excludedFactionsIds = { -- ///// unused
	--[1358] = true, -- Nat pagle
}

local colsProps = { -- total = "widthUI" -15 px
	[1] = 130,
	[2] = 30,
	[3] = 220,
	[4] = 80,
	[5] = 130,
	[6] = 110,
	[7] = 50, -- unused
}

-------------------------------------------- saved et init
REPUT_SAVED = {}

function Reput.initData(onlySettingsBoolean, subSettingsValue)
	local tbl = {
		["settings"] = {
			["version"] = addonVar["version"],
			["debug"] = 0,
			["visible"] = 1,
			["expanded"] = 1,
			["lines"] = 26, -- nombre de lignes *visibles* (suivant resize user)
			["posX"] = 300,
			["posY"] = 700,
			["showEnemy"] = false,
			["showExcluded"] = false,
			["showOnlyBest"] = false,
			["showOnlyGuild"] = false,
			["showSpecials"] = false, -- réputations "spéciales" : UNUSED dans cette version
			["sort"] = {
				["col"] = "earned",
				["asc"] = false
			},
		},
		["Alliance"] = {
			["chars"] = {}
		},
		["Horde"] = {
			["chars"] = {}
		},
	}
	
	if (Reput.isTrue(onlySettingsBoolean)) then
		if (subSettingsValue ~= nil) then return tbl["settings"][subSettingsValue] else return tbl["settings"] end
	else
		return tbl
	end
end

---------------------------------------------------------------------------------------
--------------------------------------- EVENTS ----------------------------------------
function Reput.eventHandler(self,event,...)
	if (event == "PLAYER_LOGIN") then
		Reput.loading()
	elseif (event == "PLAYER_LOGOUT") then
		Reput.updateData()
	end
end

---------------------------------------------------------------------------------------
--------------------------------- CHARGEMENT & BINDINGS -------------------------------
function Reput.loading()
	----------------------------- slash commandes
	SlashCmdList["REPUTSHOW"] = Reput.slashCmd
		SLASH_REPUTSHOW1 = "/"..addonVar["slashCmd"]
		SLASH_REPUTSHOW2 = "/reput"

	----------------------------- bindings
	
	----------------------------- settings
	if (Reput.isEmpty(REPUT_SAVED)) then
		REPUT_SAVED = Reput.initData(false)
		Reput.testPrint("=== no lua file : INIT SETTINGS ===")
	else
		local tbl = Reput.initData(true) -- only settings
		if (Reput.tableKeyDiff(tbl, REPUT_SAVED["settings"], "") ~= nil) then
			REPUT_SAVED["settings"] = Reput.copyTable(tbl, false)
			Reput.testPrint("=== diff saved data : INIT SETTINGS ===")
		end
	end

	----------------------------- suite
	playerName = UnitName("player")
	playerFaction = UnitFactionGroup("player") -- englishFaction
	playerServer = GetNormalizedRealmName()
	playerLevel = UnitLevel("player")
	local _, classFilename = UnitClass("player")
	playerClass = classFilename
	playerGuild = Reput.emptyGuild(GetGuildInfo("player"))
	
	Reput.updateMainFrame() -- complétion éléments UI
	
	if (Reput.isTrue(REPUT_SAVED["settings"]["visible"])) then
		Reput.showHide(true, true) -- scroll update
		Reput.expandUI(REPUT_SAVED["settings"]["expanded"])
	else
		Reput.showHide(false, true)
	end
	
	--------------- minimap (nécessite fichier _minimap.lua
	if (not Reput.isEmpty(addonVar["minimap"])) then
		--Reput.minimapCreate(addonVar["minimap"])
		--if (_G[addonVar["minimap"]] ~= nil) then Reput.minimapOnOff(REPUT_SAVED["settings"]["minimap"]["visible"]) end
	end
	--------------- loaded
	local str = ""
	if (not ReputFrameMain:IsVisible()) then str = " "..Reput.getLocal("Enter /reputations show to display reputations list.") end
	if (Reput.debug()) then
		SetCVar("scriptErrors", 1)
		str = str..Reput.getTxtColor(" ------ DEBUG ON.", "orange")
	end
	if (str ~= "") then Reput.echoChatAppl(Reput.getLocal("loaded")..". "..str, true, "BBBBBB") end
end

---------------------------------------------------------------------------------------
-------------------------------------- SLASH CMD --------------------------------------
function Reput.slashCmd(arg, execLevel)
	Reput.slashCmd_Lib(arg, execLevel)
end

function Reput.slashCmd_getList(arg2) -- liste des commandes-slash (voir .lib)
	-- .slashCmd_showList(dontPrint) -- print la liste en print, ou la renvoit en chaine avec dontPrint à true (pour frame cfg)
	local tbl = {
--[[
		["macro"] = {
			{
				["cmd"] = {"f_cleanline"},
				["func"] = function() Reput.cleanMarker(Reput.entierParDefaut(arg2, 1)) end,
				["label"] = "INTERNAL: pour macro text btn secure",
			},
		},
--]]
		["user"] = {
			{
				["cmd"] = {"show", "on", "off"},
				["type"] = "",
				["func"] = function() Reput.showHide() end,
				["label"] = Reput.getLocal("show/hide the list"),
			},
			{
				["cmd"] = {"hide", "exclude"},
				["type"] = "characterName",
				["func"] = function() Reput.hideChar(arg2) end,
				["label"] = Reput.getLocal("hide character"),
			},
			{
				["cmd"] = {"remove", "delete"},
				["type"] = "characterName",
				["func"] = function() Reput.deleteChar(arg2) end,
				["label"] = Reput.getLocal("delete character from Reputations list"),
			},
			{
				["cmd"] = {"add", "include"},
				["type"] = "characterName",
				["func"] = function() Reput.addChar(arg2) end,
				["label"] = Reput.getLocal("display again a character"),
			},
			{
				["cmd"] = {"reset", "resetui"},
				["type"] = "",
				["func"] = function() Reput.resetui() end,
				["label"] = Reput.getLocal("reset addon interface"),
			},
			{
				["cmd"] = {"resetall"},
				["type"] = "",
				["func"] = function() Reput.resetall() end,
				["label"] = Reput.getLocal("reset *ALL* settings to default, including saved reputations"),
			},
			{
				["cmd"] = {"help", "aide", "?"},
				["type"] = "",
				["func"] = function() Reput.help() end,
				["label"] = Reput.getLocal("display available slash commands"),
			},
		},
		["dev"] = {
			{
				["cmd"] = {"settings"},
				["func"] = function() Reput.printAddonSettings() end,
				["label"] = "display saved settings",
			},
			{
				["cmd"] = {"memory", "mem", "lines"},
				["func"] = function() Reput.printAddonMemory() end,
				["label"] = "print saved lines",
			},
			{
				["cmd"] = {"test"},
				["type"] = "string",
				["func"] = function() Reput.fcnTest(arg2) end,
				["label"] = "development test with 1 argument",
			},
			{
				["cmd"] = {"debug"},
				["type"] = "boolean",
				["func"] = function() Reput.toggleDebug() end,
				["label"] = "toggle debug (persistent)",
			},
		},
		["DEFAULTCMD"] = "help", -- commande par défaut
		["DEFAULTARG"] = "help", -- argument par défaut d'une commande
	}
	return tbl
end

---------------------------------------------------------------------------------------
--------------------------------------- ACTIONS ---------------------------------------
function Reput.help()
--[[
	local cfgFrame = Ck_cfg_frame -- /////
	local success, errorMessage = pcall(Settings.OpenToCategory, addonName)
	if (not success) then
		InterfaceOptionsFrame_OpenToCategory(cfgFrame) -- wow9
		InterfaceOptionsFrame_OpenToCategory(cfgFrame) -- wow9 (doublé pour ouvrir la page de l'addon)
	end
--]]
	Reput.slashCmd_showList(false) -- pas d'UI (mettre à true pour ne pas printer la liste)
end

function Reput.fcnTest(arg) -- $$$--%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	print(date("%y.%m.%d %H:%M:%S", GetServerTime()).."===================================")
end
	
function Reput.printAddonSettings() -- dvlpt
	Reput.echoChatAppl("settings", true, "grey")
	for key, value in pairs(REPUT_SAVED["settings"]) do
		if (type(value) == "table") then
			Reput.echo("    "..tostring(Reput.nilSiNul(key))..":", "SYSTEM")
			for subkey, subvalue in pairs(value) do
				if (type(subvalue) == "table") then value = "TABLE" end
				Reput.echo("        "..tostring(Reput.nilSiNul(subkey))..": "..tostring(Reput.nilSiNul(subvalue)), "SYSTEM")
			end
		else
			Reput.echo("    "..tostring(Reput.nilSiNul(key))..": "..tostring(Reput.nilSiNul(value)), "SYSTEM")
		end
	end
end

function Reput.printAddonMemory(brutBoolean) -- dvlpt
	if (Reput.isTrue(brutBoolean)) then
		Reput.echoTable(REPUT_SAVED[playerFaction]["chars"][playerName], playerName)
	else
		Reput.echoTable(REPUT_SAVED[playerFaction]["chars"][playerName], playerName)
--[[
		Reput.echoChatAppl("memory", true, "grey")
		for key, value in pairs(REPUT_SAVED[playerFaction]["chars"]) do
			Reput.echo("     ["..key.."]  index="..value["index"].."   builtin="..value["builtin"].."  name="..value["name"].."  cmd="..Reput.truncStr(value["cmd"], 15).."  cmdTg="..Reput.truncStr(value["cmdtarget"], 15), "SYSTEM")
		end
--]]
	end
end

function Reput.toggleDebug() -- dvlpt
	REPUT_SAVED["settings"]["debug"] = 1 - tonumber(REPUT_SAVED["settings"]["debug"])
	Reput.echoChatAppl("debug mode "..Reput.booleanToStr(Reput.debug())..".", false, "orange")
end

function Reput.resetui()
	REPUT_SAVED["settings"] = Reput.initData(true) -- init settings only
	ReputFrameMain:ClearAllPoints()
	ReputFrameMain:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", REPUT_SAVED["settings"]["posX"], REPUT_SAVED["settings"]["posY"])
	
	Reput.showHide(true)
	Reput.expandUI(true)
	Reput.scrollFrameResized(scrollFrameName, newHeight)
end

function Reput.resetall()
	Reput.confirmWarning("Reput.resetallConfirmed()")
end

function Reput.resetallConfirmed()
	wipe(REPUT_SAVED)
	REPUT_SAVED = Reput.initData(false)
	Reput.resetui()
	--if (_G[addonVar["minimap"]] ~= nil) then Reput.minimapOnOff(REPUT_SAVED["settings"]["minimap"]["visible"]) end
end

function Reput.minimapOnOff(optionalBoolean) -- UNUSED
	local val
	local frame = _G[addonVar["minimap"]]
	if (optionalBoolean == nil) then
		----------- switch
		val = (not frame:IsVisible())
	else
		----------- given value
 		val = Reput.isTrue(optionalBoolean, true)
	end
	
	if (val) then frame:Show() else frame:Hide() end
	REPUT_SAVED["settings"]["minimap"]["visible"] = Reput.booleanToInt(val)
end

function Reput.minimapIsVisible() -- renvoit true si un bouton minimap existe ET visible
	if (_G[addonVar["minimap"]] ~= nil) then
		return _G[addonVar["minimap"]]:IsVisible()
	else
		return false
	end
end

function Reput.showHide(optionalBoolean, noChatBoolean)
	local val
	if (optionalBoolean == nil) then
		----------- switch
		val = (not ReputFrameMain:IsVisible())
	else
		----------- given value
 		val = Reput.isTrue(optionalBoolean, true)
	end
	
	if (val) then
		ReputFrameMain:Show()
		Reput.setHeight(frameList, REPUT_SAVED["settings"]["lines"] * addonVar["vLine"])
		Reput.refreshList() -- effectué 2 fois si l'addon est fermé (hidden) au LOGIN
	else
		ReputFrameMain:Hide()
		if (Reput.isTrue(noChatBoolean)) then
		else
			Reput.echoChatAppl(Reput.getLocal("Enter /reputations show to display reputations list."), false, "BBBBBB")
		end
	end
	REPUT_SAVED["settings"]["visible"] = Reput.booleanToInt(val)
end

function Reput.expandUI(optionalBoolean)
	local val
	if (optionalBoolean == nil) then
		----------- switch
		val = (not frameUI:IsVisible())
	else
		----------- given value
 		val = Reput.isTrue(optionalBoolean, true)
	end
	if (val) then
		frameUI:Show()
		Reput.refreshList()
	else
		frameUI:Hide()
	end
	REPUT_SAVED["settings"]["expanded"] = Reput.booleanToInt(val)
end

function Reput.isExpanded()
	return frameUI:IsVisible()
end

function Reput.addChar(charName)
	local charName = charName or "NO_CHAR"
	if (REPUT_SAVED[playerFaction]["chars"][charName] ~= nil) then
		REPUT_SAVED[playerFaction]["chars"][charName]["excluded"] = false
		Reput.echoChatAppl(Reput.getLocal(charName.." ".."is now displayed."), false, "BBBBBB")
		Reput.refreshList()
	else
		Reput.echoChatAppl(Reput.getLocal("unable to find character").." "..charName, false, "red")
	end
end

---------------------------------------------------------------------------------------
------------------------------------ FCNS INTERNES ------------------------------------
function Reput.emptyGuild(value)
	if (value == nil) then return ("-") else return value end
end

function Reput.refreshList()
	----------- état check btns
	ReputFilterBtn_Excluded.setChecked(REPUT_SAVED["settings"]["showExcluded"])
	ReputFilterBtn_Hostiles.setChecked(REPUT_SAVED["settings"]["showEnemy"])
	ReputFilterBtn_Guild.setChecked(REPUT_SAVED["settings"]["showOnlyGuild"])
	ReputFilterBtn_Best.setChecked(REPUT_SAVED["settings"]["showOnlyBest"])
	----------- liste
	Reput.updateData() -- MAJ des reputs du perso courant : REPUT_SAVED[playerFaction]["chars"][nom_ce_joueur]
	Reput.formatData() -- formattage REPUT_SAVED[playerFaction]["chars"] pour peuplement scroll
	Reput.scrollUpdate(frameList) -- peuplement reputations list
	ReputTxtExalted_Txt:SetText(Reput.getLocal("Exalted factions:").." "..frameList.totalExalted)
end

function Reput.updateData() -- MAJ des réputs pour le perso courant (connecté)
	local dev = false -- /////
	if (dev) then print("------------"..playerName.."---------------") end
	------------------------- collecte toutes factions
	local tblReputs = {}
	local numFactions = GetNumFactions()
	local factionIndex = 1
	while (factionIndex <= numFactions) do
		local tbl = {}
		----------------- collecte data réput
		local name, _, standingId, bottomValue, topValue, earnedValue, _, _, isHeader, isCollapsed, hasRep, isWatched, _, factionID, hasBonusRepGain = GetFactionInfo(factionIndex)
		--if (excludedFactionsIds[factionID] == nil) then
			if (isHeader and isCollapsed) then
				ExpandFactionHeader(factionIndex)
				numFactions = GetNumFactions()
			end
			if (hasRep or not isHeader) then
				local standingName, tblStanding = Reput.getStanding(standingId)
				if (dev) then
					print("fIdx=", factionIndex, " -name=", name, " -id=", factionID, " -stId=", standingId, " -stName=", standingName, " -earned=", earnedValue, " -botVal=", bottomValue, " -topVal=", topValue, " -minSc=", tblStanding.scoreMin, " -maxSc=", tblStanding.scoreMax)
				end
				
				if ((earnedValue >= tblStanding.scoreMin) and (earnedValue <= tblStanding.scoreMax)) then
					----------------- faction OK
					tbl["name"] = name
					tbl["id"] = factionID
					tbl["standingId"] = standingId
					tbl["earned"] = earnedValue -- total
					tbl["bottomValue"] = bottomValue
					tbl["topValue"] = topValue
					tbl["hasBonus"] = hasBonusRepGain
					table.insert(tblReputs, tbl)
				end
			end
		--end
		factionIndex = factionIndex + 1		
	end
	if (dev) then print("---------------------------------------") end
	
	----------------- MAJ table toutes réputs
	if (REPUT_SAVED[playerFaction]["chars"][playerName] == nil) then -- perso non existant
		REPUT_SAVED[playerFaction]["chars"][playerName] = {["class"] = playerClass, ["server"] = playerServer, ["level"] = playerLevel, ["guild"] = playerGuild, ["excluded"] = false, ["reputs"] = {}}
	else --- MAJ
		REPUT_SAVED[playerFaction]["chars"][playerName]["guild"] = playerGuild
		REPUT_SAVED[playerFaction]["chars"][playerName]["level"] = playerLevel
		REPUT_SAVED[playerFaction]["chars"][playerName]["server"] = playerServer
	end
	REPUT_SAVED[playerFaction]["chars"][playerName]["reputs"] = tblReputs
end

function Reput.formatData() -- formattage REPUT_SAVED[playerFaction]["chars"] pour peuplement scroll --> frameList.data
	------------------------------- reformatage de REPUT_SAVED[playerFaction]["chars"]
	local tbl = {}
	local tblExalted = {}
	local totalExalted = 0
	
	local tblBestReputs = {}
	if (REPUT_SAVED["settings"]["showOnlyBest"]) then tblBestReputs = Reput.getBestReputs() end
--Reput.echoDebug(tblBestReputs , "____________ tblBestReputs") --/////
	for charName, tblThisChar in pairs(REPUT_SAVED[playerFaction]["chars"]) do
		if (Reput.isCharVisible(tblThisChar)) then -- filtre sur persos
			------------------- affichage OK
			for i, tblReputs in ipairs(tblThisChar.reputs) do
				if ((tblReputs.earned >= 1) or ((tblReputs.earned < 0) and (REPUT_SAVED["settings"]["showEnemy"]))) then
					---------- filtre Best reput
					local showThisRecord = true
					if (REPUT_SAVED["settings"]["showOnlyBest"]) then
						showThisRecord = (tblBestReputs[tblReputs.id].char == charName)
					end
					----------
					if (showThisRecord) then
						local currVal = tblReputs.earned - tblReputs.bottomValue
						local topVal
						if (tblReputs.topValue < tblReputs.bottomValue) then -- sécu pour réput spéciale type "paliers" du Codex des Archivistes  (id 2472)
							topVal = tblReputs.topValue - tblReputs.bottomValue
						else
							topVal = tblReputs.topValue
						end
						
						if (tblReputs.earned >= exaltedValue) then -- traitement spécial si réput à fond
							currVal = tblReputs.earned
							topVal = tblReputs.topValue
							if (tblExalted[tblReputs.id] == nil) then -- ne pas compter 1 exalté pour tous les persos
								tblExalted[tblReputs.id] = 1
								totalExalted = totalExalted + 1
							end
						end
						local tblReputChar = {
							["name"] = charName,
							["class"] = tblThisChar.class,
							["level"] = tblThisChar.level,
							["faction"] = tblReputs.name,
							["standingId"] = tblReputs.standingId,
							["earned"] = tblReputs.earned, -- pour tri
							["bottomValue"] = tblReputs.bottomValue,
							["currentValue"] = currVal,
							["topValue"] = topVal,
						}
						if (Reput.debug()) then tblReputChar.faction = tblReputChar.faction.."    ("..tblReputs.id..")" end
						table.insert(tbl, tblReputChar)
					end
				end
			end
		end
	end
	------------------------------- affectation au frameList
	frameList.totalExalted = totalExalted
	wipe(frameList.data)
	frameList.data = Reput.sortList(tbl)
end

function Reput.isCharVisible(tblChar) -- renvoit booléen visibilité *perso* suivant filtres actifs
	if (tblChar ~= nil) then
		local thisCharGuild = tblChar.guild or "" -- pour rétro-compatibilité
		local thisCharServer = tblChar.server or "" -- pour rétro-compatibilité
		
		local showThisChar = true
		if (showThisChar and tblChar.excluded and (not REPUT_SAVED["settings"]["showExcluded"])) then showThisChar = false end
		if (showThisChar and (thisCharGuild ~= playerGuild) and REPUT_SAVED["settings"]["showOnlyGuild"]) then showThisChar = false end
		return showThisChar
	else
		return true
	end
end

function Reput.getBestReputs() -- renvoit une table des persos ayant les meilleures réputations
	local tblBestReputs = {}
	
	for charName, tblThisChar in pairs(REPUT_SAVED[playerFaction]["chars"]) do
		if (Reput.isCharVisible(tblThisChar)) then
			for i, tblReputs in ipairs(tblThisChar.reputs) do
				if (tblBestReputs[tblReputs.id] ~= nil) then
					---------- reput existante
					if (tblReputs.earned > tblBestReputs[tblReputs.id].earned) then
						tblBestReputs[tblReputs.id] = {["char"] = charName, ["earned"] = tblReputs.earned}
					end
				else
					---------- reput non listée
					tblBestReputs[tblReputs.id] = {["char"] = charName, ["earned"] = tblReputs.earned}
				end
			end
		end
	end
	
	return tblBestReputs
end

function Reput.sortList(tbl) -- tri de la tbl passée par REPUT_SAVED["settings"]["sort"]["col"]
	--["sort"]["asc"] : non géré
	if (REPUT_SAVED["settings"]["sort"]["col"] == "name") then -- player_name ASC + earned DESC
		table.sort(tbl, function(a, b)
			if (a.name == b.name) then
				return (a.earned > b.earned)
			else
				return (a.name < b.name)
			end
		end)
		ReputSortBtn_Character.setChecked()
	elseif (REPUT_SAVED["settings"]["sort"]["col"] == "faction") then -- faction_name ASC + earned DESC
		table.sort(tbl, function(a, b)
			if (a.faction == b.faction) then
				return (a.earned > b.earned)
			else
				return (a.faction < b.faction)
			end
		end)
		ReputSortBtn_Faction.setChecked()
	else -- earned par défaut
		table.sort(tbl, function(a, b)
			if (a.earned == b.earned) then
				return (a.name < b.name)
			else
				return (a.earned > b.earned)
			end
		end)
		ReputSortBtn_Reputation.setChecked()
	end
	return tbl
end

function Reput.showExcludedChars() -- print liste persos excluded
	local excludedChars = ""
	for key, tblThisChar in pairs(REPUT_SAVED[playerFaction]["chars"]) do
		if (tblThisChar.excluded) then excludedChars = excludedChars.."   "..key end
	end
	if (excludedChars ~= "") then
		Reput.echoChatAppl(Reput.getLocal("Excluded characters:")..excludedChars, false, "BBBBBB")
	else
		Reput.echoChatAppl(Reput.getLocal("No excluded characters."), false, "BBBBBB")
	end
end

function Reput.hideChar(charName)
	local charName = charName or "NO_CHAR"
	if (REPUT_SAVED[playerFaction]["chars"][charName] ~= nil) then
		REPUT_SAVED[playerFaction]["chars"][charName]["excluded"] = true
		Reput.echoChatAppl(Reput.getLocal(charName.." ".."is now excluded."), false, "BBBBBB")
		Reput.refreshList()
	else
		Reput.echoChatAppl(Reput.getLocal("unable to find character").." "..charName, false, "red")
	end
end

function Reput.deleteChar(charName)
	local charName = charName or "NO_CHAR"
	if (REPUT_SAVED[playerFaction]["chars"][charName] ~= nil) then
		if (charName ~= playerName) then
			REPUT_SAVED[playerFaction]["chars"][charName] = nil
			Reput.echoChatAppl(Reput.getLocal(charName.." ".."is now removed from Reputations."), false, "BBBBBB")
			Reput.refreshList()
		else
			Reput.echoChatAppl(Reput.getLocal("cann't delete logged character"), false, "red")
		end
	else
		Reput.echoChatAppl(Reput.getLocal("unable to find character").." "..charName, false, "red")
	end
end
	
function Reput.saveSettings() -- UNUSED : save fait en direct	
end

function Reput.saveFramePosition()
	REPUT_SAVED["settings"]["posX"] = floor(ReputFrameMain:GetLeft() + 0.5)
	REPUT_SAVED["settings"]["posY"] = floor(ReputFrameMain:GetTop() + 0.5)
	Reput.testPrint("Reput.saveFramePosition: posX="..REPUT_SAVED["settings"]["posX"].." posY="..REPUT_SAVED["settings"]["posY"])
end

function Reput.saveMinimapPosition() -- UNUSED
	if (_G[addonVar["minimap"]] ~= nil) then
		--REPUT_SAVED["settings"]["minimap"]["posX"] = floor(_G[addonVar["minimap"]]:GetLeft() + 0.5)
		--REPUT_SAVED["settings"]["minimap"]["posY"] = floor(_G[addonVar["minimap"]]:GetTop() + 0.5)
	end
end

function Reput.confirmWarning(strScriptOnAccepted)
	strScriptOnAccepted = strScriptOnAccepted or nil
	
	local str = Reput.getLocal("Reset all settings to default?").."\n\n"..Reput.getTxtColor(Reput.getLocal("All of your saved reputations will be permanently deleted!"), "red")
	Reput.showAlert("|cFF"..addonVar["chatColor"]..addonVar["name"].." "..addonVar["version"].."|r\n\n\n"..str, strScriptOnAccepted)
end

function Reput.showAlert(msg, strScriptOnAccepted) -- simple alert
	StaticPopupDialogs["REPUT_ALERT"] = {
		text = "%s".."\n ",
		button1 = "Reset", -- ACCEPT,
		button2 = "No", -- CANCEL,
		OnAccept = function(self)
			--print("accepted")
			if (strScriptOnAccepted ~= nil) then RunScript(strScriptOnAccepted) end
		end,
		OnCancel = function(self)
			--print("cancelled")
		end,
		showAlert = 1, -- icon warning
		timeout = 0,
		whileDead = 1,
		hideOnEscape = 1,
		preferredIndex = 3,  -- avoid some UI taint, see http://www.wowace.com/announcements/how-to-avoid-some-ui-taint/
	};
	StaticPopup_Show("REPUT_ALERT", msg)
end

---------------------------------------------------------------------------------------
------------------------------------- LIB DEDIEE --------------------------------------
function Reput.debug() -- raccourci : renvoit true si debug ON
	return Reput.isTrue(REPUT_SAVED["settings"]["debug"])
end

function Reput.getVar(nomVar, subLvl1, subLvl2) -- nécessite addonVar
	-- get valeurs de la table locale addonVar
	-- si aucun argument n'est passé, renvoit la table addonVar
	if (addonVar == nil) then return nil end
	
	if (subLvl2 ~= nil) then
		if (addonVar[nomVar][subLvl1][subLvl2] ~= nil) then return addonVar[nomVar][subLvl1][subLvl2] end
	elseif (subLvl1 ~= nil) then
		if (addonVar[nomVar][subLvl1] ~= nil) then return addonVar[nomVar][subLvl1] end
	elseif (nomVar ~= nil) then
		if (addonVar[nomVar] ~= nil) then return addonVar[nomVar] end
	else
		return addonVar
	end
end

function Reput.setVar(nomVar, valeur) -- nécessite addonVar
	if (addonVar == nil) then return nil end
	if (nomVar ~= nil) then
		if (addonVar[nomVar] ~= nil) then addonVar[nomVar] = valeur end
	end
end

function Reput.getSavedSettings(perCharacterBoolean) -- pour appel depuis autres fichiers
	if (perCharacterBoolean) then return REPUT_SAVED else return REPUT_SAVED end
end

---------------------------------------------------------------------------------------
------------------------------------------ UI -----------------------------------------
function Reput.updateMainFrame() -- modif main frame : modif main framehidden par XML
	ReputFrameMain:SetFrameStrata("LOW")
	ReputFrameMain:SetClampedToScreen(true)
	ReputFrameMain:SetMovable(true)
 	ReputFrameMain:SetUserPlaced(true)
    ReputFrameMain:SetResizable(false)
	ReputFrameMain:ClearAllPoints()
	ReputFrameMain:SetSize(addonVar["widthUI"], 16) -- 16 px de haut /////
	ReputFrameMain:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", Reput.entierParDefaut(REPUT_SAVED["settings"]["posX"], Reput.initData(true, "posX")), Reput.entierParDefaut(REPUT_SAVED["settings"]["posY"], Reput.initData(true, "posY")))
	
	-------------------------------------------- barre titre (ReputMenuBar) qui recouvre le frameMain
	local menubar = CreateFrame("Button", "ReputMenuBar", ReputFrameMain)
	menubar:SetSize(Reput.getVar("widthUI"), 16)
	menubar:SetPoint("TOPLEFT", ReputFrameMain, "TOPLEFT", 0, 0)
	Reput.setColoredButton(menubar, {0.54,0.37,0.77}, nil, 0, Reput.getVar("alphaUI"), 0.2)
	------------- titre
	local strTitle = menubar:CreateFontString("$parent_Text", "OVERLAY", "GameFontNormalSmall")
	strTitle:SetPoint("CENTER",0,0)
	strTitle:SetText(addonVar["name"].." "..Reput.getVersion(false))
	strTitle:SetJustifyH("CENTER")
	local r, g, b = 0.92, 0.8, 0.1
	strTitle:SetTextColor(r, g, b, 1)
	------------- double-click : expand/collapse
	menubar:SetScript("OnDoubleClick", function(self, mouse)
		if (mouse == "LeftButton") then Reput.expandUI() end -- switch
	end)
	------------- drag déplacement
	menubar:SetScript("OnMouseDown", function(self, mouse)
		if (not menubar.isMoving) then
			self:GetParent():StartMoving()
			self.isMoving = true
		end
	end)
	menubar:SetScript("OnMouseUp", function(self, mouse)
		if (menubar.isMoving) then
			self:GetParent():StopMovingOrSizing()
			self.isMoving = false
			Reput.saveFramePosition()
		end
	end)
	-------------- tooltip
    menubar:SetScript("OnEnter", function(self)
		GameTooltip:SetOwner(self, "ANCHOR_TOP")
		GameTooltip:AddLine(Reput.getLocal("Double-click to collapse/expand."))
		GameTooltip:Show()
    end)
    menubar:SetScript("OnLeave", function(self)
		GameTooltip:Hide()
	end)
	
	-------------------------------------------- case de fermeture
	local btnClose = CreateFrame("Button", "ReputBtnClose", menubar)
	btnClose:SetPoint("TOPRIGHT", menubar, "TOPRIGHT", 0, 0)
	btnClose:SetSize(30, 16)
	Reput.setColoredButton(btnClose, {0.66,0.28,0.19}, nil, 0, Reput.getVar("alphaUI"), 0.2)
	-------------- texte
	local strBtn = btnClose:CreateFontString("$parent_Text", "OVERLAY", "GameFontNormalSmall")
	strBtn:SetPoint("CENTER", 1, 0)
	strBtn:SetText("X")
	strBtn:SetJustifyH("CENTER")
	------------- script
	btnClose:SetScript("OnClick", function()
		Reput.showHide(false)
	end)
	
	-------------------------------------------- frame UI (btns + scroll liste)
	frameUI = CreateFrame("Frame", "ReputUIFrame", ReputFrameMain)
	Reput.setColoredFrame(frameUI, "111111", nil, 0, 0.9, nil)
	frameUI:SetPoint("TOP", menubar, "BOTTOM", 0, 0)
	frameUI:SetWidth(addonVar["widthUI"])
	------------------------------ checkboxes filtres
	local frameBkFilter = CreateFrame("Frame", "ReputFilterFrame", frameUI)
	frameBkFilter:SetPoint("TOPLEFT", frameUI, "TOPLEFT", 0, 0)
	frameBkFilter:SetSize(addonVar["widthUI"], 25)
	Reput.createBottomBorder(frameBkFilter, 1, "444444", 0.7)
	
	local fontTbl = {["font"] = "arial", ["size"] = 11, ["color"] = "yellow"}
	local btnFilterExcluded = Reput.createCheckButton("ReputFilterBtn_Excluded", Reput.getLocal("Show excluded characters"), frameBkFilter, "444444", fontTbl)
	btnFilterExcluded:SetPoint("TOPLEFT", frameBkFilter, "TOPLEFT", 5, -5)
	btnFilterExcluded:SetScript("OnClick", function(self)
		self.setChecked(not self:getChecked())
		REPUT_SAVED["settings"]["showExcluded"] = self:getChecked()
		Reput.refreshList()
	end)
	
	local btnFilterHostiles = Reput.createCheckButton("ReputFilterBtn_Hostiles", Reput.getLocal("Show hostile factions"), frameBkFilter, "444444", fontTbl)
	btnFilterHostiles:SetPoint("LEFT", btnFilterExcluded, "RIGHT", 20, 0)
	btnFilterHostiles:SetScript("OnClick", function(self)
		self.setChecked(not self:getChecked())
		REPUT_SAVED["settings"]["showEnemy"] = self:getChecked()
		Reput.refreshList()
	end)
	
	local btnFilterGuild = Reput.createCheckButton("ReputFilterBtn_Guild", Reput.getLocal("Show only guild members"), frameBkFilter, "444444", fontTbl)
	btnFilterGuild:SetPoint("LEFT", btnFilterHostiles, "RIGHT", 20, 0)
	btnFilterGuild:SetScript("OnClick", function(self)
		self.setChecked(not self:getChecked())
		REPUT_SAVED["settings"]["showOnlyGuild"] = self:getChecked()
		Reput.refreshList()
	end)
	
	local btnFilterBest = Reput.createCheckButton("ReputFilterBtn_Best", Reput.getLocal("Show only best standing"), frameBkFilter, "444444", fontTbl)
	btnFilterBest:SetPoint("LEFT", btnFilterGuild, "RIGHT", 20, 0)
	btnFilterBest:SetScript("OnClick", function(self)
		self.setChecked(not self:getChecked())
		REPUT_SAVED["settings"]["showOnlyBest"] = self:getChecked()
		Reput.refreshList()
	end)
	
	------------------------------ boutons tri (radio)
	local frameBkSort = CreateFrame("Frame", "ReputSortFrame", frameUI)
	frameBkSort:SetPoint("TOPLEFT", frameBkFilter, "BOTTOMLEFT", 2, -2)
	frameBkSort:SetSize(200, 20) -- largeur unused
	
	frameBkSort.leftMargin = 0
	local btnSortByChar = Reput.createRadSortButton("ReputSortBtn_Character", Reput.getLocal("Sort by Character"), frameBkSort, colsProps[1] + colsProps[2] - 3)
	btnSortByChar:SetScript("OnClick", function(self)
		self.setChecked()
		REPUT_SAVED["settings"]["sort"]["col"] = "name"
		Reput.sortList(frameList.data)
		Reput.scrollUpdate(frameList)
	end)
	
	frameBkSort.leftMargin = frameBkSort.leftMargin + btnSortByChar:GetWidth() + 1
	local btnSortByFaction = Reput.createRadSortButton("ReputSortBtn_Faction", Reput.getLocal("Sort by Faction"), frameBkSort, colsProps[3] - 15)
	btnSortByFaction:SetScript("OnClick", function(self)
		self.setChecked()
		REPUT_SAVED["settings"]["sort"]["col"] = "faction"
		Reput.sortList(frameList.data)
		Reput.scrollUpdate(frameList)
	end)
	
	frameBkSort.leftMargin = frameBkSort.leftMargin + btnSortByFaction:GetWidth() + 1
	local btnSortByReputation = Reput.createRadSortButton("ReputSortBtn_Reputation", Reput.getLocal("Sort by Reputation"), frameBkSort, colsProps[4] + colsProps[5] + 12)
	btnSortByReputation:SetScript("OnClick", function(self)
		self.setChecked()
		REPUT_SAVED["settings"]["sort"]["col"] = "earned"
		Reput.sortList(frameList.data)
		Reput.scrollUpdate(frameList)
	end)
	------------------------------ popupmenu
	------------- popupmenu : frame des articles menu
	local framePopupMenu = Reput.ui_initializeMenubarPopup("ReputMenuBar_PopupMenu")
	------------- btn
	local popupTitleBtn, btnName = Reput.ui_createDownArrow(frameUI, 0.5) -- bouton fleche menu : appel par ToggleDropDownMenu()
	popupTitleBtn:SetPoint("LEFT", btnSortByReputation, "RIGHT", 10, 0)
	popupTitleBtn:SetScript("OnClick", function(self, mouse)
		ToggleDropDownMenu(1, nil, ReputMenuBar_PopupMenu, popupTitleBtn, -26, 3)
	end)
	
	------------------------------ infos
	local frameInfos = CreateFrame("Frame", "ReputTxtExalted", frameUI)
	Reput.setColoredFrame(frameInfos, nil, nil, 0, 0, nil)
	frameInfos:SetPoint("TOPRIGHT", frameUI, "TOPRIGHT", -8, -25)
	frameInfos:SetSize(100, 20)
	local strLabel = frameInfos:CreateFontString("$parent_Txt", "OVERLAY", "GameFontNormal")
	Reput.setFontStyle(strLabel, "arial", 11, "999999")
	strLabel:SetPoint("RIGHT", 0, 1)
	strLabel:SetJustifyH("RIGHT")
	strLabel:SetText("")
	
	frameUI.btnsUIheight = frameBkFilter:GetHeight() + frameBkSort:GetHeight()
	-------------------------------------------- frame des lignes (scroll)
	framePopupMenu = Reput.ui_initializeScrollPopup("ReputScroll_PopupMenu")
	Reput.ui_createUserScrollFrame(frameUI, frameUI.btnsUIheight)
	frameList.barMaxWidth = ReputScrollFrame_line_1_5:GetWidth() -- prop pour accélérer l'affichage de la bar
	
	frameUI:SetHeight(frameList:GetHeight() + frameUI.btnsUIheight)
end

function Reput.createCheckButton(btnName, label, parent, bkColor, fontTbl) 
	-- btn.getChecked disponible (propriété)
	-- btn.setChecked(boolean) disponible
	-- btn.setLabel(labelStr) disponible
	-- btn.setLabelColor(color) disponible
	local marginH = 6
	-------------------------------------------- btn (ensemble : clic sur label = clic check box)
	local btn = CreateFrame("Button", btnName, parent)
	btn:SetSize(16, 16) -- modif width plus bas
	------------------------- case à cocher
	local checkBox = CreateFrame("Frame", btnName.."_chkBox", btn) -- un Frame pour passer le clic !
	checkBox:SetSize(16, 16)
	Reput.setColoredFrame(checkBox, bkColor, "111111", 1, Reput.getVar("alphaUI"), 0.2)
	checkBox:SetPoint("LEFT", btn, "LEFT", 0, 0)
	local strCheck = checkBox:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	strCheck:SetFont(Reput.getStuffPath("arial", true), 12, "OUTLINE")
	strCheck:SetTextColor(0.86, 0.84, 0.49)
	strCheck:SetPoint("CENTER", 1, 0)
	strCheck:SetJustifyH("CENTER")
	------------------------- label
	local fLabel = CreateFrame("Button", btn:GetName().."_Label", btn)
	fLabel:SetSize(1, btn:GetHeight()) -- largeur très fine pour ne pas gêner le clic
	fLabel:SetPoint("LEFT", checkBox, "RIGHT", marginH, 0)
	------------- text
	btn.label = fLabel:CreateFontString(fLabel:GetName().."_Txt", "OVERLAY", "GameFontNormal")
	Reput.setFontStyle(btn.label, fontTbl.font, fontTbl.size, fontTbl.color)
	btn.label:SetPoint("LEFT", 0, 0)
	btn.label:SetJustifyH("LEFT")
	btn.label:SetText(label)
	-------------------------------------------- fcns
	btn.checked = false
	btn.setChecked = function(boolean)
		btn.checked = Reput.isTrue(boolean)
		strCheck:SetText("")
		if (btn.checked) then strCheck:SetText("X") end
	end
	btn.getChecked = function(boolean)
		return btn.checked
	end
	btn.setLabel = function(labelStr) btn.label:SetText(labelStr) end
	btn.setLabelColor = function(color) Reput.setFontStyle(btn.label, fontTbl.font, fontTbl.size, color) end
	btn:SetWidth(checkBox:GetWidth() + marginH + btn.label:GetStringWidth() + 4)
	-------------------------------------------- 
	return btn
end

function Reput.ui_createDownArrow(parent, hilitedAlpha) -- création du bouton d'appel d'un menu popup
	local hilitedAlpha = tonumber(hilitedAlpha) or 0.5
	
	local btnName = parent:GetName().."_popupBtn" -- besoin du nom en retour pour l'ancre du menu popup
	local popupBtn = CreateFrame("Button", btnName, parent)
	popupBtn:SetSize(16, 16)
	--popupBtn:SetFrameStrata("MEDIUM")
	popupBtn:SetNormalTexture(Reput.getStuffPath("arrow_down", true))
	popupBtn:SetHighlightTexture(Reput.getStuffPath("arrow_down", true))
	popupBtn:SetPushedTexture(Reput.getStuffPath("arrow_down", true))
	
	return popupBtn, btnName
end

function Reput.ui_initializeMenubarPopup(frameName) -- création POPUP MENU (menu bar)
	local frameName = frameName or "__PopUpMenu"
	local frame = CreateFrame("Frame", frameName, nil, "UIDropDownMenuTemplate")
	frame.displayMode = "MENU"
	frame.initialize = function(self, level)
		local menuContent = {}
		
		menuContent.notCheckable = true
		menuContent.padding = 10 -- /////
		menuContent.isTitle = nil
		menuContent.disabled = nil
		
		menuContent.text = Reput.getLocal("Refresh")
		menuContent.func = function() Reput.refreshList() end
		UIDropDownMenu_AddButton(menuContent)
		
		menuContent.text = Reput.getLocal("Display excluded characters")
		menuContent.func = function() Reput.showExcludedChars() end
		UIDropDownMenu_AddButton(menuContent)
		
		menuContent.text = Reput.getLocal("Reset addon interface")
		menuContent.func = function() Reput.resetui() end
		UIDropDownMenu_AddButton(menuContent)
		
		menuContent.text = Reput.getLocal("Reset all data...")
		menuContent.colorCode = "|cffDC571F"
		menuContent.func = function() Reput.resetall(true) end
		UIDropDownMenu_AddButton(menuContent)
	
		menuContent.text = Reput.getLocal("Display commands")
		menuContent.colorCode = "|cff66A8D9"
		menuContent.func = function() Reput.help() end
		UIDropDownMenu_AddButton(menuContent)
	
		menuContent.text = Reput.getLocal("Cancel")
		menuContent.colorCode = "|cffAAAADD"
		menuContent.func = function() end
		UIDropDownMenu_AddButton(menuContent)
	end
	
	return frame
end

function Reput.createRadSortButton(btnName, label, parent, width) -- dessin d'un bouton standard
	local bkColor = "444444"--/////
	local bkColorChecked = "5F6239"--/////
	local alpha = Reput.getVar("alphaUI")--/////
	local btnHeight = 15 --/////
	local borderColor = nil
	local borderWidth = 0
	local hilitedAlpha = 0.6
	
	local btn = CreateFrame("Button", btnName, parent)
	btn:SetSize(width, btnHeight)
	btn:SetPoint("TOPLEFT", parent, "TOPLEFT", parent.leftMargin, 0)
	Reput.setColoredButton(btn, bkColor, borderColor, borderWidth, alpha, hilitedAlpha, nil)
	-------------- texte
	local strBtn = btn:CreateFontString("$parent_Text", "OVERLAY", "GameFontNormalSmall")
	Reput.setFontStyle(strBtn, "arial", 11, "yellow")
	strBtn:SetPoint("CENTER", 1, 0)
	strBtn:SetText(label)
	strBtn:SetJustifyH("CENTER")
	-------------------------------------------- fcns
	btn.bkColor = bkColor -- couleur fond de base
	btn.setChecked = function()
		local children = {parent:GetChildren()}
		for i, child in ipairs(children) do
			Reput.setColoredButton(child, bkColor, borderColor, borderWidth, alpha, hilitedAlpha, nil)
		end
		Reput.setColoredButton(btn, bkColorChecked, borderColor, borderWidth, alpha, hilitedAlpha, nil)
		parent.checkedBtn = btn -- pour getChecked
	end
	btn.getChecked = function()
		return parent.checkedBtn
	end
	
	return btn
end

function Reput.createBottomBorder(frame, borderWidth, borderColor, borderAlpha)
	-- dessine ou modifie une bordure en bas du frame passé
	-- borderColor : nom couleur, ou couleur hexa, ou table {r, g, b}
	-- appel pour créer ou *modifier* une bordure existante : .createBorder(myFrame, 1, "blue", 0.8)
	-- pour supprimer une bordure existante : passer 0 en borderWidth
	
	local borderWidth = tonumber(borderWidth) or 0
	if (borderColor ~= nil) then
		local borderAlpha = tonumber(borderAlpha) or 1
		borderColor = Reput.getColor(borderColor, "table") -- conversion couleur en table rgb
		if (frame.border == nil) then
			----------------------------------- création bordure
			frame.border = frame:CreateLine(nil, "BACKGROUND", nil, 0)
			frame.border:SetThickness(borderWidth)
			frame.border:SetColorTexture(borderColor[1], borderColor[2], borderColor[3], borderAlpha)
			frame.border:SetStartPoint("BOTTOMRIGHT", 0, 0)
			frame.border:SetEndPoint("BOTTOMLEFT", 0, 0)
		else
			----------------------------------- modification couleur et largeur bordure
			frame.border:SetThickness(borderWidth)
			frame.border:SetColorTexture(borderColor[1], borderColor[2], borderColor[3], borderAlpha)
		end
	end
end

function Reput.ui_createUserScrollFrame(parent, marginTop) -- scroll liste reputations
	frameList = CreateFrame("Frame", "ReputScrollFrame", parent) -- définition scroll frame
	frameList:SetPoint("TOP", parent, "TOP", 0, 0 - marginTop)
	Reput.setColoredFrame(frameList, addonVar["bkColor"], nil, 0, addonVar["bkAlpha"])

	local scrollProps = { -- scroll frame properties (see _scroll.lua for full settings)
		-------------------- height: at least one of these properties must be set
		["height"] = REPUT_SAVED["settings"]["lines"] * addonVar["vLine"],
		["minHeight"] = 60, -- optional: must be set if frame's height can be resized (min = 60 px)
		["maxHeight"] = addonVar["maxVisibleLines"] * addonVar["vLine"], -- optional: must be set if frame's height can be resized
		["resizeButton"] = true, -- optional: if true, draw a resizing button at the frame's bottom
		["resizeScript"] = "Reput.scrollFrameResized", -- only needed if frame's height can be resized and some other UI elements have to be updated (NIL for no script)
		--------------------
		["width"] = addonVar["widthUI"],
		["marginH"] = 0, -- px : horizontal margin from background frame
		["marginV"] = 2, -- px : vertical margin from background frame
		["alwaysShowArrow"] = false, -- if true, always show scroller up and down arrows
		-------------------- lines
		["lineHeight"] = addonVar["vLine"], -- px
		["lineSpacing"] = 1, -- px
		["cellSpacing"] = 0, -- px // horizontal spacing, if several columns in a line
		-------------------- lines font
		["font"] = "arial",--Reput.getStuffPath("Myriad Condensed Web.ttf", true), -- "friz" | "arial" | pathToLocalFont accepted
		["fontSize"] = 11,
		["fontOutline"] = false,
		["fontColor"] = nil,
		["fontAlign"] = "LEFT", -- "LEFT" | "RIGHT" | "CENTER"
		["fontAlpha"] = 0.8,
		-------------------- lines background
		["background"] = "444444",
		["border"] = 0, -- only available if ["background"] is not a texture path
		["borderColor"] = nil, -- only available if ["background"] is not a texture path
		["alpha"] = 0.4,
		["mouseOverLineColor"] = "111111", -- visual color effect on mouseOver all the lines (nil for no mouseOver effect)
		-------------------- scroller buttons images
		["defaultScroller"] = false, -- set to true to standard WoW scroller
		["scrollerWidth"] = 16, -- px : unused if defaultScroller set to true
		["scrollerColor"] = "716BD3", -- unused if defaultScroller set to true
		["scrollerBarAlpha"] = 0.4, -- unused if defaultScroller set to true
	}
	
	local scrollColumns = { -- columns definition and properties (see _scroll.lua for full settings)
		-- column order, do not use name !
		[1] = { -- character name
			["width"] = colsProps[1],
			["padding"] = 4,
			["type"] = "Button",
			["script"] = "Reput.scrollFrameClickLineScript",
			["mouseOverLineColor"] = "625B09",
		},
		[2] = { -- level
			["width"] = colsProps[2],
			["fontAlign"] = "CENTER",
		},
		[3] = { -- faction
			["width"] = colsProps[3],
		},
		[4] = { -- standing
			["width"] = colsProps[4],
			["fontAlign"] = "CENTER",
		},
		[5] = { -- bar
			["width"] = colsProps[5],
			["background"] = "222222",
			["alpha"] = 0.5, -- alpha bk
			["border"] = 1,
			["borderColor"] = "333333",
			["template"] = "ReputBar", -- XML
			["templateType"] = "StatusBar", --XML
		},
		[6] = { -- current/top
			["width"] = colsProps[6],
			["fontAlign"] = "CENTER",
		},
		[7] = { -- earned
			["width"] = 0,
			["fontAlign"] = "RIGHT",
			["fontAlpha"] = 0.4,
		},
	}
	
	Reput.scrollCreate(frameList, scrollProps, scrollColumns, nil) -- creates structure (empty scroll lines)
end

function Reput.scrollLineUpdate(scrollFrame, lineIndex, dataLineIndex) -- do not modify args !
	local columnFrame
	--------------------------------------------------- listes reputations
	local tblReputChar = frameList.data[dataLineIndex]
	if (tblReputChar ~= nil) then -- check si resize plus grand que lignes de data
		------------------------ column 1: char name
		columnFrame = scrollFrame.line[lineIndex].column[1]
		columnFrame.text:SetText(tblReputChar.name)
		local r, g, b = GetClassColor(tblReputChar.class)
		columnFrame.text:SetTextColor(r, g, b, 1)
		------------------------ column 2: level
		columnFrame = scrollFrame.line[lineIndex].column[2]
		columnFrame.text:SetText(tblReputChar.level)
		------------------------ column 3: faction (name)
		columnFrame = scrollFrame.line[lineIndex].column[3]
		columnFrame.text:SetText(tblReputChar.faction)
		------------------------ column 4: standing
		columnFrame = scrollFrame.line[lineIndex].column[4]
		local standingName, tblStanding = Reput.getStanding(tblReputChar.standingId)
		if (Reput.debug()) then standingName = standingName.."    ("..tblReputChar.standingId..")" end
		columnFrame.text:SetText(standingName)
		columnFrame.text:SetTextColor(tblStanding.color[1], tblStanding.color[2], tblStanding.color[3], 1)
		------------------------ column 5: bar
		columnFrame = scrollFrame.line[lineIndex].column[5]
		columnFrame:SetValue(tblReputChar.currentValue * frameList.barMaxWidth / tblReputChar.topValue) -- 80 largeur
		columnFrame:SetStatusBarColor(tblStanding.color[1], tblStanding.color[2], tblStanding.color[3], 1)
		
		------------------------ column 6: current/top
		columnFrame = scrollFrame.line[lineIndex].column[6]
		
		if (tblReputChar.earned >= exaltedValue) then -- traitement spécial si réput à fond
			columnFrame.text:SetText("-")
		else
			columnFrame.text:SetText(tblReputChar.currentValue.." / "..tblReputChar.topValue)
		end
		------------------------ column 7: earned
		columnFrame = scrollFrame.line[lineIndex].column[7]
		columnFrame.text:SetText(tblReputChar.earned)
	end
end

---------------------------------------------------------------------------------------
---------------------------------- LISTE REPUTATIONS ----------------------------------
function Reput.ui_initializeScrollPopup(frameName) -- création POPUP MENU (ligne scroll emotes)
	local frameName = frameName or "___PopUpMenu2"
	local frame = CreateFrame("Frame", frameName, nil, "UIDropDownMenuTemplate")
	frame.displayMode = "MENU"
	frame.initialize = function(self, level)
		----------------------------- récupération data de la ligne cliquée
		local tblReputChar = {}
		local idxDataLine = frameList.indexLineClicked -- propriété variable du scroll frame
		if (idxDataLine ~= nil) then tblReputChar = frameList.data[idxDataLine] end
		----------------------------- construction articles menu
		if (tblReputChar ~= {}) then
			local menuContent = {}
			menuContent.padding = 10 -- /////
			menuContent.notCheckable = true

			menuContent.isTitle = true
			menuContent.text = tblReputChar.name or "  []"
			UIDropDownMenu_AddButton(menuContent)
			
			menuContent.isTitle = nil
			menuContent.disabled = nil
			
			menuContent.text = "    "..Reput.getLocal("Hide this character")
			menuContent.func = function(arg1, arg2) -- arg1 renvoit le nom du frame invisible et quelques propriétés
				Reput.ui_menuClickInLine(tblReputChar, idxDataLine, "hide") --arg1.value
			end
			UIDropDownMenu_AddButton(menuContent)
			
			if (tblReputChar.name == playerName) then menuContent.disabled = true end
			menuContent.text = "    "..Reput.getLocal("Delete from Reputations list")
			menuContent.colorCode = "|cffDC571F"
			menuContent.func = function(arg1, arg2) -- arg1 renvoit le nom du frame invisible et quelques propriétés
				Reput.ui_menuClickInLine(tblReputChar, idxDataLine, "delete") --arg1.value
			end
			UIDropDownMenu_AddButton(menuContent)
			
			menuContent.disabled = nil
			menuContent.text = Reput.getLocal("Cancel")
			menuContent.colorCode = "|cffAAAADD";
			menuContent.func = function() end
			UIDropDownMenu_AddButton(menuContent)
		end
	end
	
	return frame
end

function Reput.scrollFrameClickLineScript(scrollFrameName, clickedColumnName, mouse, lineIndex) -- do not modify args !
	-- mouse: "LeftButton" | "RightButton"
	-- if (IsModifierKeyDown()...)
	local scrollFrame = _G[scrollFrameName] -- scroll frame
	--local clickedFrame = _G[clickedColumnName] -- .text and .texture available
	local dataLineIndex, dataLine = Reput.scrollGetDataLine(scrollFrame, lineIndex) -- scroll lib
	
	if (not Reput.isEmpty(dataLine)) then
		frameList.indexLineClicked = dataLineIndex -- propriété frame scroll pour clic dans ligne avec menu local
		ToggleDropDownMenu(1, nil, ReputScroll_PopupMenu, clickedColumnName, 80, addonVar["vLine"] + 17)
	else
		Reput.echoChatAppl("unexpected error #3800", false, "red")
	end
end

function Reput.ui_menuClickInLine(tblReputChar, numLiData, actionKind)-- menu local liste emotes
--if (Reput.debug()) then Reput.echoDebug(tblReputChar , "----ui_menuClickInLine()") end --/////
	local actionKind = actionKind or "hide" -- défaut
	if (actionKind == "delete") then
		Reput.deleteChar(tblReputChar.name)
	else
		Reput.hideChar(tblReputChar.name)
	end
end

function Reput.scrollFrameResized(scrollFrameName, newHeight) -- MAJ prefs après un resize de la liste
	REPUT_SAVED["settings"]["lines"] = frameList.currentLines
	frameUI:SetHeight(frameList:GetHeight() + frameUI.btnsUIheight)
end

function Reput.getStanding(standingId) -- renvoit l'état localisé et une couleur RGB
	local standingId = Reput.strToNumber(standingId)
	local tbl = {
		[1] = {["color"] = {0.2, 0.2, 0.2}, ["scoreMin"] = -50000, ["scoreMax"] = -50000}, -- Unknown
		[2] = {["color"] = {1,0.06,0.06}, ["scoreMin"] = -36000, ["scoreMax"] = -21000}, -- Hated --?
		[3] = {["color"] = {0.95,0.42,0.29}, ["scoreMin"] = -21000, ["scoreMax"] = -9000}, -- Hostile--?
		[4] = {["color"] = {0.89,0.55,0.02}, ["scoreMin"] = -9000, ["scoreMax"] = -3000}, -- Unfriendly
		[5] = {["color"] = {0.8, 0.8, 0.8}, ["scoreMin"] = 0, ["scoreMax"] = 3000}, -- Neutral
		[6] = {["color"] = {0.82,0.87,0.13}, ["scoreMin"] = 3000, ["scoreMax"] = 9000}, -- Friendly
		[7] = {["color"] = {0.57,0.82,0.42}, ["scoreMin"] = 9000, ["scoreMax"] = 21000}, -- Honored
		[8] = {["color"] = {0.38,0.68,0.86}, ["scoreMin"] = 21000, ["scoreMax"] = 42000}, -- Revered
		[9] = {["color"] = {0.71,0.46,0.87}, ["scoreMin"] = 42000, ["scoreMax"] = 42000}, -- Exalted
	}
	return Reput.nilSiNul(_G["FACTION_STANDING_LABEL"..standingId]), tbl[standingId + 1]
	--return Reput.getStandingDEV(standingId), tbl[standingId + 1]
end

function Reput.getStandingDEV(standingId)
	local tbl = {
		[0] = "Unknown",
		[1] = "Hated",
		[2] = "Hostile",
		[3] = "Unfriendly",
		[4] = "Neutral",
		[5] = "Friendly",
		[6] = "Honored",
		[7] = "Revered",
		[8] = "Exalted",
	}
	return tbl[standingId]
end
